@@ -1,10 +1,14 @@
 from nose.plugins import Plugin
 from inspect import isclass
+import hashlib
 import os
 import sys
+import re
 import pkg_resources
 from paste.deploy import loadapp
 from pylons import config
+import unittest
+import time
 
 class CkanNose(Plugin):
     settings = None
@@ -26,7 +30,6 @@ class CkanNose(Plugin):
             # init_db is run at the start of every class because
             # when you use an in-memory sqlite db, it appears that
             # the db is destroyed after every test when you Session.Remove().
-            model.repo.init_db()
 
             ## This is to make sure the configuration is run again.
             ## Plugins use configure to make their own tables and they
@@ -36,6 +39,7 @@ class CkanNose(Plugin):
             for plugin in PluginImplementations(IConfigurable):
                 plugin.configure(config)
             
+            model.repo.init_db()
 
     def options(self, parser, env):
         parser.add_option(
@@ -53,15 +57,61 @@ class CkanNose(Plugin):
             action='store_true',
             dest='docstrings',
             help='set this to display test docstrings instead of module names')
+        parser.add_option(
+            '--segments',
+            dest='segments',
+            help='A string containing a hex digits that represent which of'
+                 'the 16 test segments to run. i.e 15af will run segments 1,5,a,f')
+
+    def wantClass(self, cls):
+        name = cls.__name__
+
+        wanted = (not cls.__name__.startswith('_')
+                  and (issubclass(cls, unittest.TestCase)
+                       or re.search('(?:^|[\b_\./-])[Tt]est', name)
+                      ))
+        
+        if self.segments and str(hashlib.md5(name).hexdigest())[0] not in self.segments:
+            return False
+
+        return wanted
+
+    def finalize(self, report):
+        if self.segments:
+            print 'Segments: %s' % self.segments
 
     def configure(self, settings, config):
         CkanNose.settings = settings
         if settings.is_ckan:
             self.enabled = True
             self.is_first_test = True
+        self.segments = settings.segments
 
     def describeTest(self, test):
         if not CkanNose.settings.docstrings:
             # display module name instead of docstring
             return False
 
+    def startTest(self, test):
+        """
+        startTest: start timing.
+        """
+##        self._started = time.time()
+
+    def stopTest(self, test):
+        """
+        stopTest: stop timing, canonicalize the test name, and save
+        the run time.
+        """
+##        runtime = time.time() - self._started
+##
+##        # CTB: HACK!
+##        f = open('times.txt', 'a')
+##
+##        testname = str(test)
+##        #if ' ' in testname:
+##        #    testname = testname.split(' ')[1]
+##
+##        f.write('%s,%s\n' % (testname, str(runtime)))
+##
+##        f.close()
