@@ -3,188 +3,77 @@ import genshi
 import datetime
 from urllib import urlencode
 
-from sqlalchemy.orm import eagerload_all
 from ckan.lib.base import BaseController, c, model, request, render, h, g
 from ckan.lib.base import ValidationException, abort, gettext
 from pylons.i18n import get_lang, _
-import ckan.authz as authz
-from ckan.authz import Authorizer
 from ckan.lib.helpers import Page
-from ckan.plugins import PluginImplementations, IGroupController, IGroupForm
 from ckan.lib.navl.dictization_functions import DataError, unflatten, validate
 from ckan.logic import NotFound, NotAuthorized, ValidationError
 from ckan.logic import check_access, get_action
-from ckan.logic.schema import group_form_schema
 from ckan.logic import tuplize_dict, clean_dict, parse_params
-from ckan.lib.dictization.model_dictize import package_dictize
 import ckan.forms
 import ckan.logic.action.get
 
+from ckan.lib.plugins import lookup_group_plugin
+
 log = logging.getLogger(__name__)
 
-# Mapping from group-type strings to IDatasetForm instances
-_controller_behaviour_for = dict()
-
-# The fallback behaviour
-_default_controller_behaviour = None
-
-def register_pluggable_behaviour(map):
-    """
-    Register the various IGroupForm instances.
-
-    This method will setup the mappings between package types and the registered
-    IGroupForm instances.  If it's called more than once an
-    exception will be raised.
-    """
-    global _default_controller_behaviour
-    
-    # Create the mappings and register the fallback behaviour if one is found.
-    for plugin in PluginImplementations(IGroupForm):
-        if plugin.is_fallback():
-            if _default_controller_behaviour is not None:
-                raise ValueError, "More than one fallback "\
-                                  "IGroupForm has been registered"
-            _default_controller_behaviour = plugin
-
-        for group_type in plugin.group_types():
-            # Create the routes based on group_type here, this will allow us to have top level
-            # objects that are actually Groups, but first we need to make sure we are not 
-            # clobbering an existing domain
-            
-            # Our version of routes doesn't allow the environ to be passed into the match call
-            # and so we have to set it on the map instead.  This looks like a threading problem
-            # waiting to happen but it is executed sequentially from instead the routing setup
-            e = map.environ
-            map.environ = {'REQUEST_METHOD': 'GET'}
-            match = map.match('/%s/new' % (group_type,))
-            map.environ = e
-            if match:
-                raise Exception, "Plugin %r would overwrite existing urls" % plugin
-            
-            map.connect('%s_new' % (group_type,), 
-                        '/%s/new' % (group_type,), controller='group', action='new')                
-            map.connect('%s_read' % (group_type,), 
-                        '/%s/{id}' %  (group_type,), controller='group', action='read')                        
-            map.connect('%s_action' % (group_type,),
-                        '/%s/{action}/{id}' % (group_type,), controller='group',
-                requirements=dict(action='|'.join(['edit', 'authz', 'history' ]))
-            )            
-            
-            if group_type in _controller_behaviour_for:
-                raise ValueError, "An existing IGroupForm is "\
-                                  "already associated with the package type "\
-                                  "'%s'" % group_type
-            _controller_behaviour_for[group_type] = plugin
-
-    # Setup the fallback behaviour if one hasn't been defined.
-    if _default_controller_behaviour is None:
-        _default_controller_behaviour = DefaultGroupForm()
-
-
-def _lookup_plugin(group_type):
-    """
-    Returns the plugin controller associoated with the given group type.
-
-    If the group type is None or cannot be found in the mapping, then the
-    fallback behaviour is used.
-    """
-    if group_type is None:
-        return _default_controller_behaviour
-    return _controller_behaviour_for.get(group_type,
-                                         _default_controller_behaviour)
-
-
-class DefaultGroupForm(object):
-    """
-    Provides a default implementation of the pluggable Group controller behaviour.
-
-    This class has 2 purposes:
-
-     - it provides a base class for IGroupForm implementations
-       to use if only a subset of the method hooks need to be customised.
-
-     - it provides the fallback behaviour if no plugin is setup to provide
-       the fallback behaviour.
-
-    Note - this isn't a plugin implementation.  This is deliberate, as
-           we don't want this being registered.
-    """
-    
-    def group_form(self):        
-        return 'group/new_group_form.html'
-
-    def form_to_db_schema(self):
-        return group_form_schema()
-
-    def db_to_form_schema(self):
-        '''This is an interface to manipulate data from the database
-        into a format suitable for the form (optional)'''
-
-
-    def check_data_dict(self, data_dict):
-        '''Check if the return data is correct, mostly for checking out if
-        spammers are submitting only part of the form
-
-        # Resources might not exist yet (eg. Add Dataset)
-        surplus_keys_schema = ['__extras', '__junk', 'state', 'groups',
-                               'extras_validation', 'save', 'return_to',
-                               'resources']
-
-        schema_keys = package_form_schema().keys()
-        keys_in_schema = set(schema_keys) - set(surplus_keys_schema)
-
-        missing_keys = keys_in_schema - set(data_dict.keys())
-
-        if missing_keys:
-            #print data_dict
-            #print missing_keys
-            log.info('incorrect form fields posted')
-            raise DataError(data_dict)
-        '''
-        pass
-
-    def setup_template_variables(self, context, data_dict):
-        c.is_sysadmin = Authorizer().is_sysadmin(c.user)
-
-        ## This is messy as auths take domain object not data_dict
-        context_group = context.get('group',None)
-        group = context_group or c.group
-        if group:
-            try:
-                if not context_group:
-                    context['group'] = group
-                check_access('group_change_state',context)
-                c.auth_for_change_state = True
-            except NotAuthorized:
-                c.auth_for_change_state = False
-
-##############      End of pluggable group behaviour     ############## 
-
-
 class GroupController(BaseController):
 
-    ## hooks for subclasses 
+    ## hooks for subclasses
 
     def _group_form(self, group_type=None):
-        return _lookup_plugin(group_type).group_form()
-        
+        return lookup_group_plugin(group_type).group_form()
+
     def _form_to_db_schema(self, group_type=None):
-        return _lookup_plugin(group_type).form_to_db_schema()
+        return lookup_group_plugin(group_type).form_to_db_schema()
 
     def _db_to_form_schema(self, group_type=None):
         '''This is an interface to manipulate data from the database
         into a format suitable for the form (optional)'''
-        return _lookup_plugin(group_type).form_to_db_schema()
+        return lookup_group_plugin(group_type).form_to_db_schema()
 
     def _setup_template_variables(self, context, data_dict, group_type=None):
-        return _lookup_plugin(group_type).setup_template_variables(context,data_dict)
+        return lookup_group_plugin(group_type).setup_template_variables(context,data_dict)
+
+    def _new_template(self,group_type):
+        return lookup_group_plugin(group_type).new_template()
+
+    def _index_template(self,group_type):
+        return lookup_group_plugin(group_type).index_template()
+
+    def _read_template(self, group_type):
+        return lookup_group_plugin(group_type).read_template()
+
+    def _history_template(self, group_type):
+        return lookup_group_plugin(group_type).history_template()
 
     ## end hooks
 
+    def _guess_group_type(self, expecting_name=False):
+        """
+            Guess the type of group from the URL handling the case
+            where there is a prefix on the URL (such as /data/organization)
+        """
+        parts = [x for x in request.path.split('/') if x]
+
+        idx = -1
+        if expecting_name:
+            idx = -2
+
+        gt = parts[idx]
+        if gt == 'group':
+            gt = None
+
+        return gt
+
+
     def index(self):
+        group_type = self._guess_group_type()
 
         context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author}
+                   'user': c.user or c.author, 'for_view': True,
+                   'with_private': False}
 
         data_dict = {'all_fields': True}
 
@@ -192,7 +81,7 @@ class GroupController(BaseController):
             check_access('site_read', context)
         except NotAuthorized:
             abort(401, _('Not authorized to see this page'))
-        
+
         results = get_action('group_list')(context, data_dict)
 
         c.page = Page(
@@ -201,15 +90,16 @@ class GroupController(BaseController):
             url=h.pager_url,
             items_per_page=20
         )
-        return render('group/index.html')
+        return render( self._index_template(group_type) )
 
 
     def read(self, id):
         from ckan.lib.search import SearchError
-        group_type = self._get_group_type(id.split('@')[0])        
+        group_type = self._get_group_type(id.split('@')[0])
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author,
-                   'schema': self._form_to_db_schema(group_type=type)}
+                   'schema': self._form_to_db_schema(group_type=group_type),
+                   'for_view': True}
         data_dict = {'id': id}
         q = c.q = request.params.get('q', '') # unicode format (decoded from utf8)
 
@@ -230,7 +120,7 @@ class GroupController(BaseController):
         except Exception, e:
             error_msg = "<span class='inline-warning'>%s</span>" % _("Cannot render description")
             c.description_formatted = genshi.HTML(error_msg)
-        
+
         c.group_admins = self.authorizer.get_admins(c.group)
 
         context['return_query'] = True
@@ -240,7 +130,7 @@ class GroupController(BaseController):
             page = int(request.params.get('page', 1))
         except ValueError, e:
             abort(400, ('"page" parameter must be an integer'))
-            
+
         # most search operations should reset the page counter:
         params_nopage = [(k, v) for k,v in request.params.items() if k != 'page']
 
@@ -254,9 +144,9 @@ class GroupController(BaseController):
             params = list(params_nopage)
             params.extend(by.items())
             return search_url(set(params))
-        
-        c.drill_down_url = drill_down_url 
-        
+
+        c.drill_down_url = drill_down_url
+
         def remove_field(key, value):
             params = list(params_nopage)
             params.remove((key, value))
@@ -281,8 +171,14 @@ class GroupController(BaseController):
                     else:
                         search_extras[param] = value
 
+
+            fq = 'capacity:"public"'
+            if (c.userobj and c.group and c.userobj.is_in_group(c.group)):
+                fq = ''
+
             data_dict = {
                 'q':q,
+                'fq':fq,
                 'facet.field':g.facets,
                 'rows':limit,
                 'start':(page-1)*limit,
@@ -299,6 +195,7 @@ class GroupController(BaseController):
                 items_per_page=limit
             )
             c.facets = query['facets']
+            c.search_facets = query['search_facets']
             c.page.items = query['results']
         except SearchError, se:
             log.error('Group search error: %r', se.args)
@@ -311,21 +208,18 @@ class GroupController(BaseController):
         c.group_activity_stream = \
                 ckan.logic.action.get.group_activity_list_html(context,
                     {'id': c.group_dict['id']})
-        
-        return render('group/read.html')
+
+        return render( self._read_template(c.group_dict['type']) )
 
     def new(self, data=None, errors=None, error_summary=None):
-        
-        group_type = request.path.strip('/').split('/')[0]
-        if group_type == 'group':
-            group_type = None
-            if data:
-                data['type'] = group_type
-        
+        group_type = self._guess_group_type(True)
+        if data:
+            data['type'] = group_type
+
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author, 'extras_as_string': True,
-                   'schema': self._form_to_db_schema(),
-                   'save': 'save' in request.params }
+                   'save': 'save' in request.params,
+                   'parent': request.params.get('parent', None)}
         try:
             check_access('group_create',context)
         except NotAuthorized:
@@ -333,7 +227,7 @@ class GroupController(BaseController):
 
         if context['save'] and not data:
             return self._save_new(context, group_type)
-        
+
         data = data or {}
         errors = errors or {}
         error_summary = error_summary or {}
@@ -341,14 +235,15 @@ class GroupController(BaseController):
 
         self._setup_template_variables(context,data)
         c.form = render(self._group_form(group_type=group_type), extra_vars=vars)
-        return render('group/new.html')
+        return render(self._new_template(group_type))
 
     def edit(self, id, data=None, errors=None, error_summary=None):
-        group_type = self._get_group_type(id.split('@')[0])                
+        group_type = self._get_group_type(id.split('@')[0])
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author, 'extras_as_string': True,
                    'save': 'save' in request.params,
-                   'schema': self._form_to_db_schema(group_type=group_type),
+                   'for_edit': True,
+                   'parent': request.params.get('parent', None)
                    }
         data_dict = {'id': id}
 
@@ -359,10 +254,6 @@ class GroupController(BaseController):
             old_data = get_action('group_show')(context, data_dict)
             c.grouptitle = old_data.get('title')
             c.groupname = old_data.get('name')
-            schema = self._db_to_form_schema()
-            if schema and not data:
-                old_data, errors = validate(old_data, schema, context=context)
-
             data = data or old_data
         except NotFound:
             abort(404, _('Group not found'))
@@ -372,7 +263,6 @@ class GroupController(BaseController):
         group = context.get("group")
         c.group = group
 
-
         try:
             check_access('group_update',context)
         except NotAuthorized, e:
@@ -387,28 +277,14 @@ class GroupController(BaseController):
 
     def _get_group_type(self, id):
         """
-        Given the id of a group it determines the plugin to load 
-        based on the group's type name (type). The plugin found
-        will be returned, or None if there is no plugin associated with 
-        the type.
-
-        Uses a minimal context to do so.  The main use of this method
-        is for figuring out which plugin to delegate to.
-
-        aborts if an exception is raised.
+        Given the id of a group it determines the type of a group given
+        a valid id/name for the group.
         """
-        global _controller_behaviour_for
-        
-        context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author}
-        try:
-            data = get_action('group_show')(context, {'id': id})
-        except NotFound:
-            abort(404, _('Group not found'))
-        except NotAuthorized:
-            abort(401, _('Unauthorized to read group %s') % id)
-        return data['type']
+        group = model.Group.get( id )
+        if not group:
+            return None
 
+        return group.type
 
     def _save_new(self, context, group_type=None):
         try:
@@ -416,8 +292,9 @@ class GroupController(BaseController):
                 tuplize_dict(parse_params(request.params))))
             data_dict['type'] = group_type or 'group'
             context['message'] = data_dict.get('log_message', '')
+            data_dict['users'] = [{'name': c.user, 'capacity': 'admin'}]
             group = get_action('group_create')(context, data_dict)
-            
+
             # Redirect to the appropriate _read route for the type of group
             h.redirect_to( group['type'] + '_read', id=group['name'])
         except NotAuthorized:
@@ -438,7 +315,7 @@ class GroupController(BaseController):
             context['message'] = data_dict.get('log_message', '')
             data_dict['id'] = id
             group = get_action('group_update')(context, data_dict)
-            h.redirect_to(controller='group', action='read', id=group['name'])
+            h.redirect_to('%s_read' % str(group['type']), id=group['name'])
         except NotAuthorized:
             abort(401, _('Unauthorized to read group %s') % id)
         except NotFound, e:
@@ -540,7 +417,7 @@ class GroupController(BaseController):
                 )
             feed.content_type = 'application/atom+xml'
             return feed.writeString('utf-8')
-        return render('group/history.html')
+        return render( self._history_template(c.group_dict['type']) )
 
     def _render_edit_form(self, fs):
         # errors arrive in c.error and fs.errors
