@@ -6,6 +6,7 @@ from datetime import datetime
 from hashlib import md5
 import logging
 import os
+import urllib
 
 from paste.deploy.converters import asbool
 from pylons import c, cache, config, g, request, response, session
@@ -16,10 +17,12 @@ from pylons.decorators import jsonify, validate
 from pylons.i18n import _, ungettext, N_, gettext
 from pylons.templating import cached_template, pylons_globals
 from genshi.template import MarkupTemplate
+from genshi.template.text import NewTextTemplate
 from webhelpers.html import literal
 
+import ckan.exceptions
 import ckan
-from ckan import authz
+import ckan.authz as authz
 from ckan.lib import i18n
 import ckan.lib.helpers as h
 from ckan.plugins import PluginImplementations, IGenshiStreamFilter
@@ -39,38 +42,110 @@ def abort(status_code=None, detail='', headers=None, comment=None):
     # #1267 Convert detail to plain text, since WebOb 0.9.7.1 (which comes
     # with Lucid) causes an exception when unicode is received.
     detail = detail.encode('utf8')
-    return _abort(status_code=status_code, 
+    return _abort(status_code=status_code,
                   detail=detail,
-                  headers=headers, 
+                  headers=headers,
                   comment=comment)
 
-def render(template_name, extra_vars=None, cache_key=None, cache_type=None, 
-           cache_expire=None, method='xhtml', loader_class=MarkupTemplate):
-    
+
+def render_snippet(template_name, **kw):
+    ''' Helper function for rendering snippets. Rendered html has
+    comment tags added to show the template used. NOTE: unlike other
+    render functions this takes a list of keywords instead of a dict for
+    the extra template variables. '''
+    # allow cache_force to be set in render function
+    cache_force = kw.pop('cache_force', None)
+    output = render(template_name, extra_vars=kw, cache_force=cache_force)
+    output = '\n<!-- Snippet %s start -->\n%s\n<!-- Snippet %s end -->\n' % (
+                    template_name, output, template_name)
+    return literal(output)
+
+def render_text(template_name, extra_vars=None, cache_force=None):
+    ''' Helper function to render a genshi NewTextTemplate without
+    having to pass the loader_class or method. '''
+    return render(template_name,
+                  extra_vars=extra_vars,
+                  cache_force=cache_force,
+                  method='text',
+                  loader_class=NewTextTemplate)
+
+def render(template_name, extra_vars=None, cache_key=None, cache_type=None,
+           cache_expire=None, method='xhtml', loader_class=MarkupTemplate,
+           cache_force = None):
+    ''' Main genshi template rendering function. '''
+
     def render_template():
         globs = extra_vars or {}
         globs.update(pylons_globals())
         globs['actions'] = model.Action
+        # add the template name to the context to help us know where we are
+        # used in depreciating functions etc
+        c.__template_name = template_name
+
+        # Using pylons.url() directly destroys the localisation stuff so
+        # we remove it so any bad templates crash and burn
+        del globs['url']
+
         template = globs['app_globals'].genshi_loader.load(template_name,
             cls=loader_class)
         stream = template.generate(**globs)
-        
+
         for item in PluginImplementations(IGenshiStreamFilter):
             stream = item.filter(stream)
-        
-        return literal(stream.render(method=method, encoding=None, strip_whitespace=False))
-    
+
+        if loader_class == NewTextTemplate:
+            return literal(stream.render(method="text", encoding=None))
+
+        return literal(stream.render(method=method, encoding=None, strip_whitespace=True))
+
+
     if 'Pragma' in response.headers:
         del response.headers["Pragma"]
-    if cache_key is not None or cache_type is not None:
-        response.headers["Cache-Control"] = "public"  
-    
-    if cache_expire is not None:
-        response.headers["Cache-Control"] = "max-age=%s, must-revalidate" % cache_expire
-    
-    return cached_template(template_name, render_template, cache_key=cache_key, 
-                           cache_type=cache_type, cache_expire=cache_expire)
-                           #, ns_options=('method'), method=method)
+
+    ## Caching Logic
+    allow_cache = True
+    # Force cache or not if explicit.
+    if cache_force is not None:
+        allow_cache = cache_force
+    # Do not allow caching of pages for logged in users/flash messages etc.
+    elif session.last_accessed:
+        allow_cache = False
+    # Tests etc.
+    elif 'REMOTE_USER' in request.environ:
+        allow_cache = False
+    # Don't cache if based on a non-cachable template used in this.
+    elif request.environ.get('__no_cache__'):
+        allow_cache = False
+    # Don't cache if we have set the __no_cache__ param in the query string.
+    elif request.params.get('__no_cache__'):
+        allow_cache = False
+    # Don't cache if we have extra vars containing data.
+    elif extra_vars:
+        for k, v in extra_vars.iteritems():
+            allow_cache = False
+            break
+    # Record cachability for the page cache if enabled
+    request.environ['CKAN_PAGE_CACHABLE'] = allow_cache
+
+    if allow_cache:
+        response.headers["Cache-Control"] = "public"
+        try:
+            cache_expire = int(config.get('ckan.cache_expires', 0))
+            response.headers["Cache-Control"] += ", max-age=%s, must-revalidate" % cache_expire
+        except ValueError:
+            pass
+    else:
+        # We do not want caching.
+        response.headers["Cache-Control"] = "private"
+        # Prevent any further rendering from being cached.
+        request.environ['__no_cache__'] = True
+
+    # Render Time :)
+    try:
+        return cached_template(template_name, render_template, loader_class=loader_class)
+    except ckan.exceptions.CkanUrlException, e:
+        raise ckan.exceptions.CkanUrlException('\nAn Exception has been raised for template %s\n%s'
+                        % (template_name, e.message))
 
 
 class ValidationException(Exception):
@@ -104,7 +179,9 @@ class BaseController(WSGIController):
         if not c.remote_addr:
             c.remote_addr = request.environ.get('REMOTE_ADDR', 'Unknown IP Address')
 
-        # what is different between session['user'] and environ['REMOTE_USER']
+        # environ['REMOTE_USER'] is set by repoze.who if it authenticates a user's
+        # cookie or OpenID. (But it doesn't check the user (still) exists in our
+        # database - we need to do that here.
         c.user = request.environ.get('REMOTE_USER', '')
         if c.user:
             c.user = c.user.decode('utf8')
@@ -116,6 +193,7 @@ class BaseController(WSGIController):
                 # and your cookie has ckan_display_name, we need to force user
                 # to logout and login again to get the User object.
                 c.user = None
+                self.log.warn('Logout to login')
         else:
             c.userobj = self._get_user_for_apikey()
             if c.userobj is not None:
@@ -130,7 +208,36 @@ class BaseController(WSGIController):
         """Invoke the Controller"""
         # WSGIController.__call__ dispatches to the Controller method
         # the request is routed to. This routing information is
-        # available in environ['pylons.routes_dict']    
+        # available in environ['pylons.routes_dict']
+
+        # Clean out any old cookies as they may contain api keys etc
+        # This also improves the cachability of our pages as cookies
+        # prevent proxy servers from caching content unless they have
+        # been configured to ignore them.
+        # we do not want to clear cookies when setting the user lang
+        if not environ.get('PATH_INFO').startswith('/user/set_lang'):
+            for cookie in request.cookies:
+                if cookie.startswith('ckan') and cookie not in ['ckan']:
+                    response.delete_cookie(cookie)
+                # Remove the ckan session cookie if not used e.g. logged out
+                elif cookie == 'ckan' and not c.user:
+                    # Check session for valid data (including flash messages)
+                    # (DGU also uses session for a shopping basket-type behaviour)
+                    is_valid_cookie_data = False
+                    for key, value in session.items():
+                        if not key.startswith('_') and value:
+                            is_valid_cookie_data = True
+                            break
+                    if not is_valid_cookie_data:
+                        if session.id:
+                            if not session.get('lang'):
+                                session.delete()
+                        else:
+                            response.delete_cookie(cookie)
+                # Remove auth_tkt repoze.who cookie if user not logged in.
+                elif cookie == 'auth_tkt' and not session.id:
+                    response.delete_cookie(cookie)
+
         try:
             return WSGIController.__call__(self, environ, start_response)
         finally:
@@ -141,8 +248,8 @@ class BaseController(WSGIController):
 
     def _set_cors(self):
         response.headers['Access-Control-Allow-Origin'] = "*"
-        response.headers['Access-Control-Allow-Methods'] = "POST, PUT, GET, DELETE"
-        response.headers['Access-Control-Allow-Headers'] = "X-CKAN-API-KEY, Content-Type"
+        response.headers['Access-Control-Allow-Methods'] = "POST, PUT, GET, DELETE, OPTIONS"
+        response.headers['Access-Control-Allow-Headers'] = "X-CKAN-API-KEY, Authorization, Content-Type"
 
     def _get_user(self, reference):
         return model.User.by_name(reference)
@@ -173,12 +280,17 @@ class BaseController(WSGIController):
         request_data = None
         if request.POST:
             try:
-                request_data = request.POST.keys()
+                keys = request.POST.keys()
+                # Parsing breaks if there is a = in the value, so for now
+                # we will check if the data is actually all in a single key
+                if keys and request.POST[ keys[0] ] in [u'1',u'']:
+                    request_data = keys[0]
+                else:
+                    request_data = urllib.unquote_plus(request.body)
             except Exception, inst:
                 msg = "Could not find the POST data: %r : %s" % \
                       (request.POST, inst)
                 raise ValueError, msg
-            request_data = request_data[0]
         else:
             try:
                 request_data = request.body
@@ -254,13 +366,13 @@ class BaseController(WSGIController):
         return path
 
     @classmethod
-    def _get_user_editable_groups(cls): 
+    def _get_user_editable_groups(cls):
         if not hasattr(c, 'user'):
             c.user = model.PSEUDO_USER__VISITOR
         import ckan.authz # Todo: Move import to top of this file?
-        groups = ckan.authz.Authorizer.authorized_query(c.user, model.Group, 
+        groups = ckan.authz.Authorizer.authorized_query(c.user, model.Group,
             action=model.Action.EDIT).all()
-        return [g for g in groups if g.state==model.State.ACTIVE] 
+        return [g for g in groups if g.state==model.State.ACTIVE]
 
     def _get_package_dict(self, *args, **kwds):
         import ckan.forms
@@ -319,7 +431,7 @@ class BaseController(WSGIController):
             update_or_add = 'add'
         else:
             user_or_authgroup = None
-            update_or_add = None            
+            update_or_add = None
 
         # Work out what role checkboxes are checked or unchecked
         checked_roles = [ box_id for (box_id, value) in request.params.items() \
