@@ -1,11 +1,13 @@
 import logging
 from ckan.lib.base import _
 import ckan.authz
-import ckan.new_authz as new_authz
+from ckan.new_authz import is_authorized
 from ckan.lib.navl.dictization_functions import flatten_dict, DataError
 from ckan.plugins import PluginImplementations
 from ckan.plugins.interfaces import IActions
 
+log = logging.getLogger(__name__)
+
 class AttributeDict(dict):
     def __getattr__(self, name):
         try:
@@ -23,6 +25,11 @@ class ActionError(Exception):
     def __init__(self, extra_msg=None):
         self.extra_msg = extra_msg
 
+    def __str__(self):
+        err_msgs = (super(ActionError, self).__str__(),
+                    self.extra_msg)
+        return ' - '.join([str(err_msg) for err_msg in err_msgs if err_msg])
+
 class NotFound(ActionError):
     pass
 
@@ -32,12 +39,18 @@ class NotAuthorized(ActionError):
 class ParameterError(ActionError):
     pass
 
+
 class ValidationError(ParameterError):
     def __init__(self, error_dict, error_summary=None, extra_msg=None):
         self.error_dict = error_dict
         self.error_summary = error_summary
         self.extra_msg = extra_msg
 
+    def __str__(self):
+        err_msgs = (super(ValidationError, self).__str__(),
+                    self.error_summary)
+        return ' - '.join([str(err_msg) for err_msg in err_msgs if err_msg])
+
 log = logging.getLogger(__name__)
 
 def parse_params(params, ignore_keys=None):
@@ -138,7 +151,7 @@ def check_access(action, context, data_dict=None):
         #    # TODO Check the API key is valid at some point too!
         #    log.debug('Valid API key needed to make changes')
         #    raise NotAuthorized
-        logic_authorization = new_authz.is_authorized(action, context, data_dict)
+        logic_authorization = is_authorized(action, context, data_dict)
         if not logic_authorization['success']:
             msg = logic_authorization.get('msg','')
             raise NotAuthorized(msg)
@@ -212,3 +225,29 @@ def get_action(action):
     _actions.update(fetched_actions)
     return _actions.get(action)
 
+def get_or_bust(data_dict, keys):
+    '''Try and get values from dictionary and if they are not there
+    raise a validataion error.
+
+    data_dict: a dictionary
+    keys: either a single string key in which case will return a single value,
+    or a iterable which will return a tuple for unpacking purposes.
+
+    e.g single_value = get_or_bust(data_dict, 'a_key')
+        value_1, value_2 = get_or_bust(data_dict, ['key1', 'key2'])
+    '''
+    values = []
+    errors = {}
+
+    if isinstance(keys, basestring):
+        keys = [keys]
+    for key in keys:
+        value = data_dict.get(key)
+        if not value:
+            errors[key] = _('Missing value')
+        values.append(value)
+    if errors:
+        raise ValidationError(errors)
+    if len(values) == 1:
+        return values[0]
+    return tuple(values)
