@@ -1,24 +1,33 @@
-from ckan.logic import check_access_old, NotFound
+import ckan.logic as logic
 from ckan.authz import Authorizer
 from ckan.lib.base import _
 
-
 def package_create(context, data_dict=None):
     model = context['model']
     user = context['user']
-
-    check1 = check_access_old(model.System(), model.Action.PACKAGE_CREATE, context)
+    check1 = logic.check_access_old(model.System(), model.Action.PACKAGE_CREATE, context)
 
     if not check1:
         return {'success': False, 'msg': _('User %s not authorized to create packages') % str(user)}
     else:
-        
-        check2 = check_group_auth(context,data_dict)
+
+        check2 = _check_group_auth(context,data_dict)
         if not check2:
             return {'success': False, 'msg': _('User %s not authorized to edit these groups') % str(user)}
 
     return {'success': True}
 
+def related_create(context, data_dict=None):
+    model = context['model']
+    user = context['user']
+    userobj = model.User.get( user )
+
+    if userobj:
+        return {'success': True}
+
+    return {'success': False, 'msg': _('You must be logged in to add a related item')}
+
+
 def resource_create(context, data_dict):
     return {'success': False, 'msg': 'Not implemented yet in the auth refactor'}
 
@@ -34,7 +43,7 @@ def package_relationship_create(context, data_dict):
     authorized = Authorizer().\
                     authorized_package_relationship(\
                     user, pkg1, pkg2, action=model.Action.EDIT)
-    
+
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to edit these packages') % str(user)}
     else:
@@ -43,8 +52,8 @@ def package_relationship_create(context, data_dict):
 def group_create(context, data_dict=None):
     model = context['model']
     user = context['user']
-   
-    authorized = check_access_old(model.System(), model.Action.GROUP_CREATE, context)
+
+    authorized = logic.check_access_old(model.System(), model.Action.GROUP_CREATE, context)
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to create groups') % str(user)}
     else:
@@ -53,8 +62,8 @@ def group_create(context, data_dict=None):
 def authorization_group_create(context, data_dict=None):
     model = context['model']
     user = context['user']
-   
-    authorized = check_access_old(model.System(), model.Action.AUTHZ_GROUP_CREATE, context)
+
+    authorized = logic.check_access_old(model.System(), model.Action.AUTHZ_GROUP_CREATE, context)
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to create authorization groups') % str(user)}
     else:
@@ -67,14 +76,15 @@ def rating_create(context, data_dict):
 def user_create(context, data_dict=None):
     model = context['model']
     user = context['user']
-   
-    authorized = check_access_old(model.System(), model.Action.USER_CREATE, context)
+
+    authorized = logic.check_access_old(model.System(), model.Action.USER_CREATE, context)
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to create users') % str(user)}
     else:
         return {'success': True}
 
-def check_group_auth(context, data_dict):
+
+def _check_group_auth(context, data_dict):
     if not data_dict:
         return True
 
@@ -83,7 +93,7 @@ def check_group_auth(context, data_dict):
 
     api_version = context.get('api_version') or '1'
 
-    group_blobs = data_dict.get("groups", []) 
+    group_blobs = data_dict.get("groups", [])
     groups = set()
     for group_blob in group_blobs:
         # group_blob might be a dict or a group_ref
@@ -98,7 +108,7 @@ def check_group_auth(context, data_dict):
             id = group_blob
         grp = model.Group.get(id)
         if grp is None:
-            raise NotFound(_('Group was not found.'))
+            raise logic.NotFound(_('Group was not found.'))
         groups.add(grp)
 
     if pkg:
@@ -107,7 +117,7 @@ def check_group_auth(context, data_dict):
         groups = groups - set(pkg_groups)
 
     for group in groups:
-        if not check_access_old(group, model.Action.EDIT, context):
+        if not logic.check_access_old(group, model.Action.EDIT, context):
             return False
 
     return True
@@ -130,6 +140,14 @@ def group_create_rest(context, data_dict):
 
     return group_create(context, data_dict)
 
+def vocabulary_create(context, data_dict):
+    user = context['user']
+    return {'success': Authorizer.is_sysadmin(user)}
+
 def activity_create(context, data_dict):
     user = context['user']
     return {'success': Authorizer.is_sysadmin(user)}
+
+def tag_create(context, data_dict):
+    user = context['user']
+    return {'success': Authorizer.is_sysadmin(user)}
