@@ -1,6 +1,7 @@
 from nose.tools import assert_equal 
 
 from ckan import model
+from ckan.lib.create_test_data import CreateTestData
 
 from ckan.tests.functional.api.base import BaseModelApiTestCase
 from ckan.tests.functional.api.base import Api1TestCase as Version1TestCase 
@@ -9,8 +10,15 @@ from ckan.tests.functional.api.base import ApiUnversionedTestCase as Unversioned
 
 class RevisionsTestCase(BaseModelApiTestCase):
 
-    commit_changesets = False
-    reuse_common_fixtures = True
+    @classmethod
+    def setup_class(cls):
+        CreateTestData.create()
+        cls.user_name = u'annafan' # created in CreateTestData
+        cls.init_extra_environ(cls.user_name)
+
+    @classmethod
+    def teardown_class(cls):
+        model.repo.rebuild_db()
     
     def test_register_get_ok(self):
         # Comparison list - newest first
@@ -26,7 +34,7 @@ class RevisionsTestCase(BaseModelApiTestCase):
         assert_equal(revs_result, [rev.id for rev in revs])
 
     def test_entity_get_ok(self):
-        rev = model.repo.history().all()[-2] # 2nd revision is the creation of pkgs
+        rev = model.repo.history().all()[0] # newest revision is the creation of pkgs
         assert rev.id
         assert rev.timestamp.isoformat()
         offset = self.revision_offset(rev.id)
