@@ -6,6 +6,9 @@ from ckan.plugins import SingletonPlugin, implements, IGroupController
 from ckan import plugins
 import ckan.model as model
 from ckan.lib.create_test_data import CreateTestData
+from ckan.logic import check_access, NotAuthorized
+
+from pylons import config
 
 from ckan.tests import *
 from ckan.tests import setup_test_search_index
@@ -14,11 +17,11 @@ from ckan.tests import search_related, is_search_supported
 
 class MockGroupControllerPlugin(SingletonPlugin):
     implements(IGroupController)
-    
+
     def __init__(self):
         from collections import defaultdict
         self.calls = defaultdict(int)
-    
+
     def read(self, entity):
         self.calls['read'] += 1
 
@@ -37,6 +40,10 @@ class MockGroupControllerPlugin(SingletonPlugin):
     def delete(self, entity):
         self.calls['delete'] += 1
 
+    def before_view(self, data_dict):
+        self.calls['before_view'] += 1
+        return data_dict
+
 class TestGroup(FunctionalTestCase):
 
     @classmethod
@@ -60,12 +67,12 @@ class TestGroup(FunctionalTestCase):
         assert 'Groups' in res, res
         assert 'Groups</a>' in res, res
         res = res.click(href='/group', index=0)
-        assert 'Groups of' in res, res
+        assert "Dave's books" in res, res
 
     def test_index(self):
         offset = url_for(controller='group', action='index')
         res = self.app.get(offset)
-        assert '<h1 class="page_heading">Groups' in res, res
+        assert re.search('<h1(.*)>\s*Groups', res.body)
         groupname = 'david'
         group = model.Group.by_name(unicode(groupname))
         group_title = group.title
@@ -74,7 +81,7 @@ class TestGroup(FunctionalTestCase):
         self.check_named_element(res, 'tr', group_title, group_packages_count, group_description)
         res = res.click(group_title)
         assert groupname in res
-        
+
     def test_read_non_existent(self):
         name = u'group_does_not_exist'
         offset = url_for(controller='group', action='read', id=name)
@@ -86,7 +93,7 @@ class TestGroup(FunctionalTestCase):
         name = u'david'
         offset = url_for(controller='group', action='read', id=name)
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
-        assert plugin.calls['read'] == 2, plugin.calls
+        assert plugin.calls['read'] == 1, plugin.calls
         plugins.unload(plugin)
 
     def test_read_and_authorized_to_edit(self):
@@ -186,7 +193,7 @@ Ho ho ho
         res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
         # should be read page
         # assert 'Groups - %s' % self.groupname in res, res
-        
+
         model.Session.remove()
         group = model.Group.by_name(self.groupname)
         assert group.title == newtitle, group
@@ -211,11 +218,11 @@ Ho ho ho
         user = model.User.by_name(u'russianfan')
         model.setup_default_user_roles(pkg, [user])
         model.repo.commit_and_remove()
-        
+
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'annakarenina' in res, res
         assert 'newone' in res
-        
+
     def test_4_new_duplicate_package(self):
         prefix = ''
 
@@ -252,6 +259,20 @@ Ho ho ho
         assert plugin.calls['edit'] == 1, plugin.calls
         plugins.unload(plugin)
 
+    def test_edit_image_url(self):
+        group = model.Group.by_name(self.groupname)
+        offset = url_for(controller='group', action='edit', id=self.groupname)
+        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+
+        form = res.forms['group-edit']
+        image_url = u'http://url.to/image_url'
+        form['image_url'] = image_url
+        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+
+        model.Session.remove()
+        group = model.Group.by_name(self.groupname)
+        assert group.image_url == image_url, group
+
     def test_edit_non_existent(self):
         name = u'group_does_not_exist'
         offset = url_for(controller='group', action='edit', id=name)
@@ -262,14 +283,14 @@ Ho ho ho
         CreateTestData.create_groups([{'name': group_name,
                                        'packages': [self.packagename]}],
                                      admin_user_name='russianfan')
-                                       
+
         group = model.Group.by_name(group_name)
         offset = url_for(controller='group', action='edit', id=group_name)
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
         main_res = self.main_div(res)
         assert 'Edit: %s' % group.title in main_res, main_res
         assert 'value="active" selected' in main_res, main_res
-        
+
         # delete
         form = res.forms['group-edit']
         form['state'] = 'deleted'
@@ -288,7 +309,7 @@ class TestNew(FunctionalTestCase):
     def setup_class(self):
         model.Session.remove()
         CreateTestData.create()
-        
+
         self.packagename = u'testpkg'
         model.repo.new_revision()
         model.Session.add(model.Package(name=self.packagename))
@@ -367,9 +388,9 @@ class TestNew(FunctionalTestCase):
         fv[prefix+'name'] = group_name
         res = fv.submit('save', status=200, extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'Group name already exists' in res, res
-        self.check_tag(res, '<form', 'class="has-errors"')
+        self.check_tag(res, '<form', 'has-errors')
         assert 'class="field_error"' in res, res
-    
+
     def test_new_plugin_hook(self):
         plugin = MockGroupControllerPlugin()
         plugins.load(plugin)
@@ -412,7 +433,7 @@ class TestRevisions(FunctionalTestCase):
             grp.description = self.description[i]
             model.repo.commit_and_remove()
 
-        self.grp = model.Group.by_name(self.name)        
+        self.grp = model.Group.by_name(self.name)
 
     @classmethod
     def teardown_class(self):
@@ -452,3 +473,252 @@ class TestRevisions(FunctionalTestCase):
         assert 'xmlns="http://www.w3.org/2005/Atom"' in res, res
         assert '</feed>' in res, res
 
+
+class TestOrganizationGroup(FunctionalTestCase):
+
+    @classmethod
+    def setup_class(self):
+        model.Session.remove()
+        CreateTestData.create(auth_profile='publisher')
+
+    @classmethod
+    def teardown_class(self):
+        model.repo.rebuild_db()
+
+    def test_index(self):
+        from pylons import config
+        from nose.exc import SkipTest
+        if config.get('ckan.auth.profile', '') != 'publisher':
+            raise SkipTest('Publisher auth profile not enabled')
+
+        offset = url_for(controller='group', action='index')
+        res = self.app.get(offset)
+        assert '<h1 class="page_heading">Groups' in res, res
+        groupname = 'david'
+        group = model.Group.by_name(unicode(groupname))
+        group_title = group.title
+        group_packages_count = len(group.active_packages().all())
+        group_description = group.description
+        self.check_named_element(res, 'tr', group_title, group_packages_count, group_description)
+        res = res.click(group_title)
+        assert groupname in res
+        assert 'organization' == group.type, group.type
+
+    def test_read(self):
+        from pylons import config
+        from nose.exc import SkipTest
+        if config.get('ckan.auth.profile', '') != 'publisher':
+            raise SkipTest('Publisher auth profile not enabled')
+
+        # Relies on the search index being available
+        setup_test_search_index()
+        name = u'david'
+        title = u'Dave\'s books'
+        pkgname = u'warandpeace'
+        group = model.Group.by_name(name)
+        assert 'organization' == group.type
+        for group_ref in (group.name, group.id):
+            offset = url_for(controller='group', action='read', id=group_ref)
+            res = self.app.get(offset)
+            main_res = self.main_div(res)
+            assert title in res, res
+            assert 'Administrators' in res, res
+            assert 'russianfan' in main_res, main_res
+            assert name in res, res
+            assert '0 datasets found.' in self.strip_tags(main_res), main_res
+
+    def test_read_and_not_authorized_to_edit(self):
+        from pylons import config
+        from nose.exc import SkipTest
+        if config.get('ckan.auth.profile', '') != 'publisher':
+            raise SkipTest('Publisher auth profile not enabled')
+
+        name = u'david'
+        title = u'Dave\'s books'
+        pkgname = u'warandpeace'
+        offset = url_for(controller='group', action='edit', id=name)
+        res = self.app.get(offset, extra_environ={'REMOTE_USER': 'russianfan'}, status=200)
+
+
+class TestPublisherEdit(FunctionalTestCase):
+
+    @classmethod
+    def setup_class(self):
+        from ckan.tests.mock_publisher_auth import MockPublisherAuth
+        self.auth = MockPublisherAuth()
+
+        model.Session.remove()
+        CreateTestData.create(auth_profile='publisher')
+        self.groupname = u'david'
+        self.packagename = u'testpkg'
+        model.repo.new_revision()
+        model.Session.add(model.Package(name=self.packagename))
+        model.repo.commit_and_remove()
+
+    @classmethod
+    def teardown_class(self):
+        model.Session.remove()
+        model.repo.rebuild_db()
+        model.Session.remove()
+
+
+    def test_0_not_authz(self):
+        from pylons import config
+        from nose.exc import SkipTest
+        if config.get('ckan.auth.profile', '') != 'publisher':
+            raise SkipTest('Publisher auth profile not enabled')
+
+        offset = url_for(controller='group', action='edit', id=self.groupname)
+        # 401 gets caught by repoze.who and turned into redirect
+        res = self.app.get(offset, status=[302, 401])
+        res = res.follow()
+        assert res.request.url.startswith('/user/login')
+
+    def test_2_edit(self):
+        from pylons import config
+        from nose.exc import SkipTest
+        if config.get('ckan.auth.profile', '') != 'publisher':
+            raise SkipTest('Publisher auth profile not enabled')
+
+        group = model.Group.by_name(self.groupname)
+        offset = url_for(controller='group', action='edit', id=self.groupname)
+        user = model.User.get('russianfan')
+
+        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        assert 'Edit: %s' % group.title in res, res
+
+        form = res.forms['group-edit']
+        titlefn = 'title'
+        descfn = 'description'
+        newtitle = 'xxxxxxx'
+        newdesc = '''### Lots of stuff here
+
+Ho ho ho
+'''
+
+        form[titlefn] = newtitle
+        form[descfn] = newdesc
+        pkg = model.Package.by_name(self.packagename)
+        form['packages__2__name'] = pkg.name
+
+
+        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        # should be read page
+        # assert 'Groups - %s' % self.groupname in res, res
+
+        model.Session.remove()
+        group = model.Group.by_name(self.groupname)
+        assert group.title == newtitle, group
+        assert group.description == newdesc, group
+
+        # now look at datasets
+        assert len(group.active_packages().all()) == 3
+
+    def test_3_edit_form_has_new_package(self):
+        # check for dataset in autocomplete
+        offset = url_for(controller='package', action='autocomplete', q='an')
+        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        assert 'annakarenina' in res, res
+        assert not 'newone' in res, res
+        model.repo.new_revision()
+        pkg = model.Package(name=u'anewone')
+        model.Session.add(pkg)
+        model.repo.commit_and_remove()
+
+        model.repo.new_revision()
+        pkg = model.Package.by_name(u'anewone')
+        user = model.User.by_name(u'russianfan')
+        model.setup_default_user_roles(pkg, [user])
+        model.repo.commit_and_remove()
+
+        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        assert 'annakarenina' in res, res
+        assert 'newone' in res
+
+    def test_4_new_duplicate_package(self):
+        prefix = ''
+
+        # Create group
+        group_name = u'testgrp4'
+        CreateTestData.create_groups([{'name': group_name,
+                                       'packages': [self.packagename]}],
+                                     admin_user_name='russianfan')
+
+        # Add same package again
+        offset = url_for(controller='group', action='edit', id=group_name)
+        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        fv = res.forms['group-edit']
+        fv['packages__1__name'] = self.packagename
+        res = fv.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = res.follow()
+        assert group_name in res, res
+        model.Session.remove()
+
+        # check package only added to the group once
+        group = model.Group.by_name(group_name)
+        pkg_names = [pkg.name for pkg in group.active_packages().all()]
+        assert_equal(pkg_names, [self.packagename])
+
+    def test_edit_plugin_hook(self):
+        plugin = MockGroupControllerPlugin()
+        plugins.load(plugin)
+        offset = url_for(controller='group', action='edit', id=self.groupname)
+        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        form = res.forms['group-edit']
+        group = model.Group.by_name(self.groupname)
+        form['title'] = "huhuhu"
+        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        assert plugin.calls['edit'] == 1, plugin.calls
+        plugins.unload(plugin)
+
+    def test_edit_non_auth(self):
+        offset = url_for(controller='group', action='edit', id=self.groupname)
+        res = self.app.get(offset, status=[302,401], extra_environ={'REMOTE_USER': 'non-existent'})
+
+    def test_edit_fail_auth(self):
+        context = { 'group': model.Group.by_name(self.groupname), 'model': model, 'user': 'russianfan' }
+        try:
+            if self.auth.check_access('group_update',context, {}):
+                assert False, "Check access said we were allowed but we shouldn't really"
+        except NotAuthorized, e:
+            pass # Do nothing as this is what we expected
+
+    def test_edit_success_auth(self):
+        userobj = model.User.get('russianfan')
+        grp = model.Group.by_name(self.groupname)
+
+        def gg(*args, **kwargs):
+            return [grp]
+        model.User.get_groups = gg
+
+        context = { 'group': grp, 'model': model, 'user': 'russianfan' }
+        try:
+            self.auth.check_access('group_update',context, {})
+        except NotAuthorized, e:
+            assert False, "The user should have access"
+
+
+    def test_delete(self):
+        group_name = 'deletetest'
+        CreateTestData.create_groups([{'name': group_name,
+                                       'packages': [self.packagename]}],
+                                     admin_user_name='russianfan')
+
+        group = model.Group.by_name(group_name)
+        offset = url_for(controller='group', action='edit', id=group_name)
+        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        main_res = self.main_div(res)
+        assert 'Edit: %s' % group.title in main_res, main_res
+        assert 'value="active" selected' in main_res, main_res
+
+        # delete
+        form = res.forms['group-edit']
+        form['state'] = 'deleted'
+        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+
+        group = model.Group.by_name(group_name)
+        assert_equal(group.state, 'deleted')
+        res = self.app.get(offset, status=302)
+        res = res.follow()
+        assert res.request.url.startswith('/user/login'), res.request.url
+
